


#[User]
set             User_Advanced "True"
set           User_Adjustment "False"

#[About]
set               About_Title "About"
set             About_LogoGif "logo.gif"
set           About_Copyright "copyright.txt"
set             About_Profile "info.txt"
set                  About_OK "OK"

#[Main]
set                Main_Title "TouchCfg"
set              Main_LogoGif "logo.gif"
set            Main_Interface "Interface"
set                 Main_Type "Type"
set            Main_FWVersion "Version"
set               Main_Status "Status"
set       Main_PanelCandidate "Panel Candidate List"
set             Main_AddPanel "Add"
set          Main_RescanPanel "Rescan"
set          Main_RemovePanel "Remove"
set                   Main_OK "OK"
set              Main_Testing "Testing..."
set         Main_E_noresponce "E (no responce)"
set           Main_E_notfound "E (not found)"
set            Main_E_notchar "E (not char device)"
set         Main_E_openfailed "E (open failed)"
set            Main_E_unknown "E (?)"
set                Main_Sound "Sound"
set              Main_NoSound "No Sound"
set       Main_SoundTouchDown "Touch Down"
set         Main_SoundTouchUp "Touch Up"
set            Main_MouseMode "Mouse Mode"
set              Main_Desktop "Desktop Mode"
set              Main_Drawing "Drawing Mode"
set           Main_ModeButton "Mode Button"
set     Main_DoubleClickSpeed "Dbl-Click Speed:"
set      Main_DoubleClickArea "Dbl-Click Area:"
set             Main_SlowDown "<< Slower"
set              Main_SpeedUp "Faster >>"
set           Main_ShrinkArea "<< Smaller"
set           Main_ExpandArea "Larger >>"
set             Main_DrawTest "Draw Test"
set               Main_Cal4Pt "4-Pt Cal"
set                Main_About "About"
set                 Main_Quit "Close"
set             Main_Advanced "Advanced >>"

#[AddPanel]
set            AddPanel_Title "Add Panel"
set        AddPanel_Interface "Interface:"
set               AddPanel_OK "OK"
set           AddPanel_Cancel "Cancel"
set       AddPanel_E_notfound "<file not found>"
set        AddPanel_E_notchar "<not char device>"
set          AddPanel_E_exist "<already in list>"

#[Advanced]
set            Advanced_Title "Advanced Settings"
set          Advanced_Cal25Pt "25-Pt Coef"
set        Advanced_Calibrate "Calibrate"
set            Advanced_Clear "Clear"
set         Advanced_PerfTune "Perf Tuning"
set       Advanced_ParameterA "Parameter A"
set       Advanced_ParameterB "Parameter B"
set       Advanced_ParameterC "Parameter C"
set       Advanced_ParameterD "Parameter D"
set       Advanced_ParameterE "Parameter E"
set          Advanced_Default "Default"
set            Advanced_Apply "Apply"
set            Advanced_Close "Close"

#[Exec]
set                Exec_Title "Exec Command"
set                   Exec_OK "OK"

#[ModeButton]
set          ModeButton_Mode0 "Drawing"
set          ModeButton_Mode1 "Desktop"


