#ifndef _CONFIGINT_H_
#define _CONFIGINT_H_

#ifndef FIFO_MODE
#define            FIFO_MODE   420
#endif
#ifndef AP2D_SETEEPROM
#define       AP2D_SETEEPROM   7189
#endif
#ifndef AP2D_CMD_BASE
#define        AP2D_CMD_BASE   7191
#endif
#ifndef EPR_DefaultFilterOn
#define  EPR_DefaultFilterOn   0
#endif
#ifndef TouchKit_USB_VID
#define     TouchKit_USB_VID   14371
#endif
#ifndef AP2D_SETMODE
#define         AP2D_SETMODE   7180
#endif
#ifndef AP2D_SETSOUNDMODE
#define    AP2D_SETSOUNDMODE   7181
#endif
#ifndef SCR_MAX_MNR
#define          SCR_MAX_MNR   2
#endif
#ifndef AP2D_RESET
#define           AP2D_RESET   7178
#endif
#ifndef AP2D_RCLICKON
#define        AP2D_RCLICKON   7184
#endif
#ifndef TM_DRAWING
#define           TM_DRAWING   0
#endif
#ifndef AP2D_SUICIDE
#define         AP2D_SUICIDE   7179
#endif
#ifndef DRAW_NAME
#define            DRAW_NAME   drawtest
#endif
#ifndef DefaultDblClickArea
#define  DefaultDblClickArea   30
#endif
#ifndef AP2D_SHAREPORT
#define       AP2D_SHAREPORT   7173
#endif
#ifndef _LeftButton
#define          _LeftButton   1
#endif
#ifndef TPANELD_NAME
#define         TPANELD_NAME   tpaneld
#endif
#ifndef AP2D_SETDEBUG
#define        AP2D_SETDEBUG   7176
#endif
#ifndef MinRClickTool
#define        MinRClickTool   0
#endif
#ifndef AP2D_SETDCSPEED
#define      AP2D_SETDCSPEED   7182
#endif
#ifndef SM_UP
#define                SM_UP   2
#endif
#ifndef MinDblClickArea
#define      MinDblClickArea   15
#endif
#ifndef SCR_BASE_MNR
#define         SCR_BASE_MNR   180
#endif
#ifndef AP2D_RCLICK
#define          AP2D_RCLICK   7177
#endif
#ifndef RCLICK_DELAY
#define         RCLICK_DELAY   2
#endif
#ifndef SM_NONE
#define              SM_NONE   0
#endif
#ifndef MaxDblClickArea
#define      MaxDblClickArea   50
#endif
#ifndef USBMODULE_NAME
#define       USBMODULE_NAME   tkusb
#endif
#ifndef EPR_DefaultFast
#define      EPR_DefaultFast   36
#endif
#ifndef AP2D_RCLICKDELAYON
#define   AP2D_RCLICKDELAYON   7186
#endif
#ifndef AP2D_RCLICKCANCEL
#define    AP2D_RCLICKCANCEL   7188
#endif
#ifndef EPR_DefaultADLimit
#define   EPR_DefaultADLimit   50
#endif
#ifndef TouchKit_MODE
#define        TouchKit_MODE   420
#endif
#ifndef TouchKit_USB_PID
#define     TouchKit_USB_PID   1
#endif
#ifndef AP2D_RCLICKOFF
#define       AP2D_RCLICKOFF   7185
#endif
#ifndef AP2D_SENDDATA
#define        AP2D_SENDDATA   7175
#endif
#ifndef EPR_DefaultLiftCount
#define EPR_DefaultLiftCount   0
#endif
#ifndef _4PCAL_NAME
#define          _4PCAL_NAME   4pcal
#endif
#ifndef DefaultMouseMode
#define     DefaultMouseMode   0
#endif
#ifndef PRJNAME
#define              PRJNAME   TouchKit
#endif
#ifndef MinDblClickSpeed
#define     MinDblClickSpeed   28
#endif
#ifndef V_btAdvancedOn
#define       V_btAdvancedOn   1
#endif
#ifndef DefaultDblClickSpeed
#define DefaultDblClickSpeed   18
#endif
#ifndef EPR_DefaultAccLimit
#define  EPR_DefaultAccLimit   256
#endif
#ifndef AP2D_XMODULEREADY
#define    AP2D_XMODULEREADY   7190
#endif
#ifndef TM_DESKTOP
#define           TM_DESKTOP   1
#endif
#ifndef TOUCHCFG_NAME
#define        TOUCHCFG_NAME   touchcfg
#endif
#ifndef EPR_DefaultDebounced
#define EPR_DefaultDebounced   7
#endif
#ifndef USBPNPD_NAME
#define         USBPNPD_NAME   usbpnpd
#endif
#ifndef XDRIVER_NAME
#define         XDRIVER_NAME   touchkit
#endif
#ifndef _MidButton
#define           _MidButton   2
#endif
#ifndef EPR_DefaultLiftRange
#define EPR_DefaultLiftRange   20
#endif
#ifndef AP2D_FLUSHOUTPUT
#define     AP2D_FLUSHOUTPUT   7187
#endif
#ifndef _RightButton
#define         _RightButton   3
#endif
#ifndef AP2D_SHOWSTATUS
#define      AP2D_SHOWSTATUS   7172
#endif
#ifndef EPR_DefaultPowerSaveOn
#define EPR_DefaultPowerSaveOn   0
#endif
#ifndef MaxRClickTool
#define        MaxRClickTool   1
#endif
#ifndef _25PCAL_NAME
#define         _25PCAL_NAME   25pcal
#endif
#ifndef FrameSize
#define            FrameSize   5
#endif
#ifndef PerfTuningOn
#define         PerfTuningOn   0
#endif
#ifndef SM_DOWN
#define              SM_DOWN   1
#endif
#ifndef MaxDblClickSpeed
#define     MaxDblClickSpeed   8
#endif
#ifndef EPR_DefaultSlow
#define      EPR_DefaultSlow   10
#endif
#ifndef AP2D_CLOSEPORT
#define       AP2D_CLOSEPORT   7174
#endif
#ifndef AP2D_SETDCAREA
#define       AP2D_SETDCAREA   7183
#endif
#ifndef DefaultRClickTool
#define    DefaultRClickTool   0
#endif
#ifndef DefaultSoundMode
#define     DefaultSoundMode   0
#endif
#ifndef AP2D_GETMODE
#define         AP2D_GETMODE   7191
#endif

#endif

