#ifndef _CONFIGSTR_H_
#define _CONFIGSTR_H_

#ifndef _DblClickSpeed
#define       _DblClickSpeed   "DblClickSpeed"
#endif
#ifndef RCLICK_LOCK
#define          RCLICK_LOCK   "/tmp/rclick.lock"
#endif
#ifndef TouchKit_CONF
#define        TouchKit_CONF   "/etc/tpaneld.conf"
#endif
#ifndef FIFO_MODE
#define            FIFO_MODE   "420"
#endif
#ifndef AP2D_SETEEPROM
#define       AP2D_SETEEPROM   "7189"
#endif
#ifndef TouchKit_DAT
#define         TouchKit_DAT   "/dev/tkdat"
#endif
#ifndef TouchKit_LOCK
#define        TouchKit_LOCK   "/var/run/tpaneld.pid"
#endif
#ifndef AP2D_CMD_BASE
#define        AP2D_CMD_BASE   "7191"
#endif
#ifndef XMODULE_HOME
#define         XMODULE_HOME   "/usr/X11R6/lib/modules/"
#endif
#ifndef EPR_DefaultFilterOn
#define  EPR_DefaultFilterOn   "0"
#endif
#ifndef TouchKit_USB_VID
#define     TouchKit_USB_VID   "14371"
#endif
#ifndef AP2D_SETMODE
#define         AP2D_SETMODE   "7180"
#endif
#ifndef _Sound
#define               _Sound   "Sound"
#endif
#ifndef AP_LINE
#define              AP_LINE   "yellow"
#endif
#ifndef AP2D_SETSOUNDMODE
#define    AP2D_SETSOUNDMODE   "7181"
#endif
#ifndef KeywordList
#define          KeywordList   "MouseMode DblClickSpeed DblClickArea RClickTool Port Sound"
#endif
#ifndef XI_CONTROL
#define           XI_CONTROL   "PCTRL"
#endif
#ifndef SCR_MAX_MNR
#define          SCR_MAX_MNR   "2"
#endif
#ifndef AP2D_RESET
#define           AP2D_RESET   "7178"
#endif
#ifndef AP_TOUCH_DOWN
#define        AP_TOUCH_DOWN   "green"
#endif
#ifndef AP2D_RCLICKON
#define        AP2D_RCLICKON   "7184"
#endif
#ifndef TM_DRAWING
#define           TM_DRAWING   "0"
#endif
#ifndef AP2D_SUICIDE
#define         AP2D_SUICIDE   "7179"
#endif
#ifndef DefaultConfData
#define      DefaultConfData   "\
 MouseMode = DRAWING \n\r\
 DblClickSpeed = 18\n\r\
 DblClickArea = 30\n\r\
 RClickTool = 0\n\r\
 Sound = 0\n\r\
 Port = /dev/ttyS0\n\r\
 Port = /dev/ttyS1\n\r\
 Port = /dev/psaux\n\r\
 Port = /dev/tkpanel0\n\r\
 Port = /dev/tkpanel1\n\r\
 Port = /dev/tkpanel0\n\r\
 Port = /dev/tkpanel1\n\r\
"
#endif
#ifndef sTM_DESKTOP
#define          sTM_DESKTOP   "DESKTOP"
#endif
#ifndef TouchKit_CMDIN
#define       TouchKit_CMDIN   "/dev/tkcmdin"
#endif
#ifndef AP_CURSOR
#define            AP_CURSOR   "gumby"
#endif
#ifndef TouchKit_SHRIN
#define       TouchKit_SHRIN   "/dev/tkshrin"
#endif
#ifndef TouchKit_CMD
#define         TouchKit_CMD   "/dev/tkcmd"
#endif
#ifndef DRAW_NAME
#define            DRAW_NAME   "drawtest"
#endif
#ifndef DefaultDblClickArea
#define  DefaultDblClickArea   "30"
#endif
#ifndef AP2D_SHAREPORT
#define       AP2D_SHAREPORT   "7173"
#endif
#ifndef _LeftButton
#define          _LeftButton   "1"
#endif
#ifndef TPANELD_NAME
#define         TPANELD_NAME   "tpaneld"
#endif
#ifndef DefaultDeviceList
#define    DefaultDeviceList   "/dev/ttyS0 /dev/ttyS1 /dev/psaux /dev/tkpanel0 /dev/tkpanel1 /dev/tkpanel0 /dev/tkpanel1"
#endif
#ifndef ModeList
#define             ModeList   "DRAWING DESKTOP"
#endif
#ifndef AP2D_SETDEBUG
#define        AP2D_SETDEBUG   "7176"
#endif
#ifndef sSM_DOWN
#define             sSM_DOWN   "Down"
#endif
#ifndef MinRClickTool
#define        MinRClickTool   "0"
#endif
#ifndef AP2D_SETDCSPEED
#define      AP2D_SETDCSPEED   "7182"
#endif
#ifndef DRIVER_NAME
#define          DRIVER_NAME   "touchkit"
#endif
#ifndef SM_UP
#define                SM_UP   "2"
#endif
#ifndef MinDblClickArea
#define      MinDblClickArea   "15"
#endif
#ifndef MODULE_NAME
#define          MODULE_NAME   "TouchKit"
#endif
#ifndef SCR_BASE_MNR
#define         SCR_BASE_MNR   "180"
#endif
#ifndef AP2D_RCLICK
#define          AP2D_RCLICK   "7177"
#endif
#ifndef RCLICK_DELAY
#define         RCLICK_DELAY   "2"
#endif
#ifndef USBPORT
#define              USBPORT   "/dev/tkpanel"
#endif
#ifndef SM_NONE
#define              SM_NONE   "0"
#endif
#ifndef MaxDblClickArea
#define      MaxDblClickArea   "50"
#endif
#ifndef _DblClickArea
#define        _DblClickArea   "DblClickArea"
#endif
#ifndef XI_PANEL
#define             XI_PANEL   "TKPANEL"
#endif
#ifndef USB_PID_MORE
#define         USB_PID_MORE   "0001 0001 0002 0001 0002"
#endif
#ifndef USBMODULE_NAME
#define       USBMODULE_NAME   "tkusb"
#endif
#ifndef EPR_DefaultFast
#define      EPR_DefaultFast   "36"
#endif
#ifndef AP2D_RCLICKDELAYON
#define   AP2D_RCLICKDELAYON   "7186"
#endif
#ifndef AP2D_RCLICKCANCEL
#define    AP2D_RCLICKCANCEL   "7188"
#endif
#ifndef TOUCHCFG_CMD
#define         TOUCHCFG_CMD   "/tmp/touchcfg.cmd"
#endif
#ifndef DefaultUSBpnpdConf
#define   DefaultUSBpnpdConf   "\
#Device = VendorID, ProductID, ActionIfPlugIn\
Device = 0x3823, 0x0001, insmod tkusb; /usr/bin/tpaneld reset\n\r\
"
#endif
#ifndef EPR_DefaultADLimit
#define   EPR_DefaultADLimit   "50"
#endif
#ifndef TouchKit_MODE
#define        TouchKit_MODE   "420"
#endif
#ifndef TouchKit_USB_PID
#define     TouchKit_USB_PID   "1"
#endif
#ifndef USBPNPD_ACTION
#define       USBPNPD_ACTION   "insmod tkusb; /usr/bin/tpaneld reset"
#endif
#ifndef AP2D_RCLICKOFF
#define       AP2D_RCLICKOFF   "7185"
#endif
#ifndef SoundModeList
#define        SoundModeList   "None Down Up"
#endif
#ifndef DEV_LIST
#define             DEV_LIST   "/proc/bus/usb/devices"
#endif
#ifndef AP2D_SENDDATA
#define        AP2D_SENDDATA   "7175"
#endif
#ifndef TouchKit_DEVICENAME
#define  TouchKit_DEVICENAME   "tpanel"
#endif
#ifndef PNP_CONFIG
#define           PNP_CONFIG   "/etc/usbpnpd.conf"
#endif
#ifndef EPR_DefaultLiftCount
#define EPR_DefaultLiftCount   "0"
#endif
#ifndef _4PCAL_NAME
#define          _4PCAL_NAME   "4pcal"
#endif
#ifndef DefaultMouseMode
#define     DefaultMouseMode   "0"
#endif
#ifndef _Port
#define                _Port   "Port"
#endif
#ifndef _RClickTool
#define          _RClickTool   "RClickTool"
#endif
#ifndef PRJNAME
#define              PRJNAME   "TouchKit"
#endif
#ifndef MinDblClickSpeed
#define     MinDblClickSpeed   "28"
#endif
#ifndef AP_CROSS_WAIT
#define        AP_CROSS_WAIT   "blue"
#endif
#ifndef ROM_FORMAT
#define           ROM_FORMAT   "%s.rom"
#endif
#ifndef V_btAdvancedOn
#define       V_btAdvancedOn   "1"
#endif
#ifndef DefaultDblClickSpeed
#define DefaultDblClickSpeed   "18"
#endif
#ifndef AP_BACKGROUND
#define        AP_BACKGROUND   "black"
#endif
#ifndef TouchKit_ROMHOME
#define     TouchKit_ROMHOME   "/etc/tpanel.rom/"
#endif
#ifndef EPR_DefaultAccLimit
#define  EPR_DefaultAccLimit   "256"
#endif
#ifndef AP2D_XMODULEREADY
#define    AP2D_XMODULEREADY   "7190"
#endif
#ifndef TM_DESKTOP
#define           TM_DESKTOP   "1"
#endif
#ifndef TOUCHCFG_NAME
#define        TOUCHCFG_NAME   "touchcfg"
#endif
#ifndef EPR_DefaultDebounced
#define EPR_DefaultDebounced   "7"
#endif
#ifndef USBPNPD_NAME
#define         USBPNPD_NAME   "usbpnpd"
#endif
#ifndef XDRIVER_NAME
#define         XDRIVER_NAME   "touchkit"
#endif
#ifndef _MidButton
#define           _MidButton   "2"
#endif
#ifndef sDefaultSoundMode
#define    sDefaultSoundMode   "0"
#endif
#ifndef TouchKit_CMDOUT
#define      TouchKit_CMDOUT   "/dev/tkcmdout"
#endif
#ifndef USB_VID_MORE
#define         USB_VID_MORE   "0123 3823 3823 0EEF 0EEF"
#endif
#ifndef sTM_DRAWING
#define          sTM_DRAWING   "DRAWING"
#endif
#ifndef EPR_DefaultLiftRange
#define EPR_DefaultLiftRange   "20"
#endif
#ifndef PRG_LOCK
#define             PRG_LOCK   "/var/run/usbpnpd.pid"
#endif
#ifndef AP2D_FLUSHOUTPUT
#define     AP2D_FLUSHOUTPUT   "7187"
#endif
#ifndef _RightButton
#define         _RightButton   "3"
#endif
#ifndef sSM_NONE
#define             sSM_NONE   "None"
#endif
#ifndef TouchKit_USBMODULE
#define   TouchKit_USBMODULE   "TouchKit"
#endif
#ifndef AP_CROSS_OK
#define          AP_CROSS_OK   "yellow"
#endif
#ifndef TouchKit_HOME
#define        TouchKit_HOME   "/usr/local/TouchKit/"
#endif
#ifndef AP2D_SHOWSTATUS
#define      AP2D_SHOWSTATUS   "7172"
#endif
#ifndef PANEL_SECTION_NAME
#define   PANEL_SECTION_NAME   "touchkitpanel"
#endif
#ifndef XDRV_HOME
#define            XDRV_HOME   "/usr/X11R6/lib/modules/input/"
#endif
#ifndef EPR_DefaultPowerSaveOn
#define EPR_DefaultPowerSaveOn   "0"
#endif
#ifndef MaxRClickTool
#define        MaxRClickTool   "1"
#endif
#ifndef AP_GRID
#define              AP_GRID   "green"
#endif
#ifndef _25PCAL_NAME
#define         _25PCAL_NAME   "25pcal"
#endif
#ifndef FrameSize
#define            FrameSize   "5"
#endif
#ifndef AP_TOUCH_UP
#define          AP_TOUCH_UP   "red"
#endif
#ifndef TouchKit_SHROUT
#define      TouchKit_SHROUT   "/dev/tkshrout"
#endif
#ifndef PerfTuningOn
#define         PerfTuningOn   "0"
#endif
#ifndef SM_DOWN
#define              SM_DOWN   "1"
#endif
#ifndef MaxDblClickSpeed
#define     MaxDblClickSpeed   "8"
#endif
#ifndef EPR_DefaultSlow
#define      EPR_DefaultSlow   "10"
#endif
#ifndef AP2D_CLOSEPORT
#define       AP2D_CLOSEPORT   "7174"
#endif
#ifndef AP2D_SETDCAREA
#define       AP2D_SETDCAREA   "7183"
#endif
#ifndef DefaultRClickTool
#define    DefaultRClickTool   "0"
#endif
#ifndef DefaultSoundMode
#define     DefaultSoundMode   "0"
#endif
#ifndef AP2D_GETMODE
#define         AP2D_GETMODE   "7191"
#endif
#ifndef _MouseMode
#define           _MouseMode   "MouseMode"
#endif
#ifndef sDefaultMode
#define         sDefaultMode   "DRAWING"
#endif
#ifndef sSM_UP
#define               sSM_UP   "Up"
#endif
#ifndef TouchKit_SHR
#define         TouchKit_SHR   "/dev/tkshr"
#endif

#endif

